/*
 * GQoob
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */


#include "gqoob.h"
#include "display_xine.h"

#include "display.h"
#include "io_xine.h"
#include "ui2_button.h"
#include "ui2_dial.h"
#include "ui2_item.h"
#include "ui2_list.h"
#include "ui2_main.h"
#include "ui2_number.h"
#include "ui2_overlay.h"
#include "ui2_slider.h"
#include "ui2_text.h"
#include "ui2_widget.h"
#include "utils_xine.h"


/*
 *-----------------------------------------------------------------------------
 * xine specific controls
 *-----------------------------------------------------------------------------
 */

void display_xine_speed_update(void)
{
	gint speed;

	speed = gqoob_xine_speed_get();
	item_section_set("speed", main_window, gqoob_xine_speed_simple(speed));
}

static void overlay_video_size_cb(OverlayData *od, const gchar *key, gint hidden, gpointer data)
{
	if (main_window && od->skin->ui != main_window) return;
	
	if (debug_mode) printf("overlay size is %d x %d at %d, %d (hidden:%d)\n", od->width, od->height, od->x, od->y, hidden);

	gqoob_xine_move(od->x, od->y, od->width, od->height, hidden);
}

static void button_xine_fullscreen_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	btn_fullscreen_pressed();
}

static void button_xine_fullscreen_controls_cb(ButtonData *button, const gchar *key, gpointer data)
{
	btn_fullscreen_controls_toggle_pressed();
}

static void button_xine_speed_up_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	btn_speed_up_pressed();
}

static void button_xine_speed_down_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	btn_speed_down_pressed();
}

static void button_xine_skip_30_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	skip_by_time(30);
}

static void button_xine_skip_30back_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	skip_by_time(-30);
}

static void button_xine_skip_60_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	skip_by_time(60);
}

static void button_xine_skip_60back_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	skip_by_time(-60);
}

static void button_xine_skip_replay_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	skip_by_time(-5);
}

static void button_xine_skip_value_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	gint val;
	const gchar *text;

	text = ui_widget_get_data(button->wd, "data");
	if (text)
		{
		val = strtol(text, NULL, 10);
		skip_by_time(val);
		}
}

static gint item_xine_speed_status_cb(ItemData *item, const gchar *key, gpointer data)
{
	gint speed;

	speed = gqoob_xine_speed_get();
	return gqoob_xine_speed_simple(speed);
}

void display_xine_register_widgets(UIData *ui)
{
	/* xine specific */

	overlay_register_key("video", ui,
			     overlay_video_size_cb, NULL);

	button_register_key("fullscreen", ui,
			    NULL, NULL,
			    button_xine_fullscreen_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("fullscreen.controls", ui,
			    NULL, NULL,
			    button_xine_fullscreen_controls_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);

	button_register_key("speed.up", ui,
			    NULL, NULL,
			    button_xine_speed_up_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("speed.down", ui,
			    NULL, NULL,
			    button_xine_speed_down_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);

	button_register_key("skip.30", ui,
			    NULL, NULL,
			    button_xine_skip_30_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("skip.30back", ui,
			    NULL, NULL,
			    button_xine_skip_30back_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("skip.60", ui,
			    NULL, NULL,
			    button_xine_skip_60_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("skip.60back", ui,
			    NULL, NULL,
			    button_xine_skip_60back_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("skip.replay", ui,
			    NULL, NULL,
			    button_xine_skip_replay_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("skip.value", ui,
			    NULL, NULL,
			    button_xine_skip_value_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);

	item_register_key("speed", ui, item_xine_speed_status_cb, NULL);
}

