/*
 * GQoob
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */


#include "gqoob.h"
#include "io_mpg123.h"

#include "ui_fileops.h"
#include "ui_utildlg.h"


/* ----------------------------------------------------------
   this was the location of input / output interface to mpg123
   but for xine interface it was stripped down to only the
   header and id3 utilities. for origin please see GQmpeg.
   ----------------------------------------------------------*/


typedef struct {
        gint read_id3;
} Mpg123ExtraData;


	/* options */
gint mpg123_read_id3_tags = TRUE;


/*
 *----------------------------------------------------------------------------
 * mpg123 module extra data
 *----------------------------------------------------------------------------
 */

static void mpg123_extra_data_free(gpointer data)
{
	Mpg123ExtraData *xd = data;

	if (!xd) return;

	g_free(xd);
}

static Mpg123ExtraData *mpg123_extra_data_new(SongData *sd)
{
	Mpg123ExtraData *xd;

	if (!sd) return NULL;

	if (sd->data) return sd->data;

	xd = g_new0(Mpg123ExtraData, 1);
	xd->read_id3 = FALSE;

	sd->data = xd;
	sd->free_data_func = mpg123_extra_data_free;

	return xd;
}

/*
 *----------------------------------------------------------------------------
 * mpg123 module callback funcs
 *----------------------------------------------------------------------------
 */

void mpg123_data_init(SongData *sd)
{
	sd->type_description = _("MPEG audio file");
	mpg123_extra_data_new(sd);
}

gint mpg123_data_set(SongData *sd, gint read_info)
{
	if (read_info)
		{
		Mpg123ExtraData *xd = mpg123_extra_data_new(sd);

		if (mpg123_read_id3_tags && !xd->read_id3)
			{
			ID3_Data *idd;

			idd = get_id3_tag(sd->path);
			if (idd)
				{
				sd->title = idd->title;
				sd->artist = idd->artist;
				sd->album = idd->album;
				sd->year = idd->year;
				sd->comment = idd->comment;
				sd->genre = idd->genre_description;
				sd->track = idd->track;

				g_free(idd); /* don't free members */
				}
			xd->read_id3 = TRUE;
			}

		if (!sd->info_loaded)
			{
			Mpg_Data *md;

			md = get_mpg_header_info(sd->path);
			if (md)
				{
				sd->length = md->length;
				sd->live = FALSE;
				sd->bit_rate = md->bit_rate;
				sd->bit_depth = 16;
				sd->khz_rate = md->sample_rate;
				if (md->stereo)
					{
					sd->channels = 2;
					}
				else
					{
					sd->channels = 1;
					}

				g_free(md);
				}
			else
				{
				sd->flags |= SONG_FLAG_NOT_FOUND;
				}
			}
		sd->info_loaded = TRUE;
		}

	return TRUE;
}

/*
 *----------------------------------------------------------------------------
 * mpg123 module http edit funcs
 *----------------------------------------------------------------------------
 */
#if 0
static GtkWidget *mpg123_http_entry_setup(const gchar *path)
{
	GtkWidget *vbox;
	GtkWidget *entry;

	vbox = gtk_vbox_new(FALSE, 0);

	entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(entry), "http://");
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
	gtk_widget_show(entry);

	gtk_object_set_data(GTK_OBJECT(vbox), "http_URL", entry);

	return vbox;
}

static GtkWidget *mpg123_http_edit(SongData *sd)
{
	GtkWidget *vbox;
	GtkWidget *entry;

	vbox = gtk_vbox_new(FALSE, 0);

	entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(entry), sd->path);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
	gtk_widget_show(entry);

	gtk_object_set_data(GTK_OBJECT(vbox), "http_URL", entry);

	return vbox;
}

static gchar *mpg123_http_get_path(GtkWidget *widget)
{
	GtkWidget *entry = gtk_object_get_data(GTK_OBJECT(widget), "http_URL");

	if (!entry) return NULL;

	return g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
}
#endif

