/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "gqoob.h"
#include "skin_default.h"

#include "ui2_button.h"
#include "ui2_item.h"
#include "ui2_list.h"
#include "ui2_number.h"
#include "ui2_overlay.h"
#include "ui2_slider.h"
#include "ui2_skin.h"
#include "ui2_text.h"
#include "ui2_util.h"
#include "ui2_widget.h"


#include "default_skin/back.xpm"
#include "default_skin/back_mini.xpm"
#include "default_skin/btn_exit.xpm"
#include "default_skin/btn_iconify.xpm"
#include "default_skin/btn_mnext.xpm"
#include "default_skin/btn_mode.xpm"
#include "default_skin/btn_mpause.xpm"
#include "default_skin/btn_mplay.xpm"
#include "default_skin/btn_mprev.xpm"
#include "default_skin/btn_mstop.xpm"
#include "default_skin/btn_next.xpm"
#include "default_skin/btn_pause.xpm"
#include "default_skin/btn_play.xpm"
#include "default_skin/btn_prev.xpm"
#include "default_skin/btn_sizer.xpm"
#include "default_skin/btn_stop.xpm"
#include "default_skin/dig_lrg.xpm"
#include "default_skin/dig_sml.xpm"
#include "default_skin/focus.xpm"

#include "default_skin/flyby_back.xpm"
#include "default_skin/btn_snext.xpm"
#include "default_skin/btn_sprev.xpm"

#include "default_skin/menu_back.xpm"
#include "default_skin/menu_list_back.xpm"
#include "default_skin/menu_list_row.xpm"
#include "default_skin/menu_list_icons.xpm"


#define GQOOB_SKIN_FONT "-*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*"


#if 0
static WidgetData *add_item(const gchar *key, gchar **data, gint sections,
			    gint x, gint y, SkinData *skin, const gchar *text_id)
{
	ItemData *item;

	item = item_new_from_data(data, x, y, sections);
	return item_register(skin, item, key, text_id);
}
#endif

static DigitData *add_digit(const gchar *key, gchar **data, SkinData *skin, const gchar *text_id)
{
	DigitData *digit;

	digit = digit_new_from_data(data);
	digit_register(skin, digit, key, text_id);
	digit_unref(digit);

	return digit;
}

static WidgetData *add_number(const gchar *key, gint length, gint rotation, gint zeros, gint centered, gint x, gint y,
			      DigitData *digit, SkinData *skin, const gchar *text_id)
{
	NumberData *number;

	number = number_new(digit, x, y, length, zeros, centered, rotation);
	return number_register(skin, number, key, text_id);
}

static WidgetData *add_button(const gchar *key, gchar **data, gint prelight, gint light, gint x, gint y,
			      gchar **clip_data, SkinData *skin, const gchar *text_id)
{
	ButtonData *button;

	button = button_new_from_data(data, x, y, prelight, light, clip_data);
	return button_register(skin, button, key, text_id);
}

#if 0
static WidgetData *add_slider(const gchar *key, gchar **data, gint has_press, gint prelight, gint vertical, gint reversed,
			      gint length, gint x, gint y, SkinData *skin, const gchar *text_id,
			      gint sizeable, gint real_length, gint border_1, gint border_2, gint stretch)
{
	SliderData *slider;

	slider = slider_new_from_data(data, x, y, length, vertical, reversed, has_press, prelight, NULL);
	if (sizeable) slider_set_attributes(slider, sizeable, real_length, border_1, border_2, stretch);
	return slider_register(skin, slider, key, text_id);
}
#endif

static WidgetData *add_text(const gchar *key, const gchar *font, gint width, gint sizeable, gint rotation,
			    guint8 r, guint8 g, guint8 b, guint8 a,
			    gint x, gint y, SkinData *skin, const gchar *text_id)
{
	TextData *text;

	text = text_new_from_x(font, x, y, width, sizeable, rotation);
	text_set_color(text, r, g, b, a);
	return text_register(skin, text, key, text_id);
}

/* double size utilities */
static void double_window(SkinData *skin)
{
	util_size(&skin->width);
	util_size(&skin->height);
	util_size(&skin->border_left);
	util_size(&skin->border_right);
	util_size(&skin->border_bottom);
	util_size(&skin->border_top);
}

static void double_focus(SkinData *skin)
{
	util_size(&skin->focus_border_left);
	util_size(&skin->focus_border_right);
	util_size(&skin->focus_border_bottom);
	util_size(&skin->focus_border_top);
}

static void add_focus(SkinData *skin)
{
	skin->focus_overlay = util_size_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)focus_xpm), TRUE);
	skin->focus_box_alpha = 150;
	skin->focus_has_border = TRUE;
	skin->focus_border_left = 5;
	skin->focus_border_right = 5;
	skin->focus_border_bottom = 5;
	skin->focus_border_top = 5;
}

static void finish_simple_window(SkinData *skin)
{
	skin->width = gdk_pixbuf_get_width(skin->real_overlay);
	skin->height = gdk_pixbuf_get_height(skin->real_overlay);

	/* this is merely to that the skin editor has sane values for these */
	skin->width_def = skin->width_min = skin->width_max = skin->width;
	skin->height_def = skin->height_min = skin->height_max = skin->height;
}

SkinData *skin_load_default_cb(UIData *ui, const gchar *key, gpointer data)
{
	SkinData *skin;
	DigitData *ds, *dl;
	WidgetData *wd;

	skin = skin_new();

	if (key && strcmp(key, "skindata_fullscreen") == 0)
		{
		/* small mode */
		skin->real_overlay = util_size_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)back_mini_xpm), TRUE);

		add_text("title", GQOOB_SKIN_FONT, 128, FALSE, 0, 255, 255, 255, 179, 6, 4, skin, NULL);

		add_button("pause", (gchar **)btn_mpause_xpm, TRUE, TRUE, 211, 4, NULL, skin, NULL);
		add_button("play", (gchar **)btn_mplay_xpm, TRUE, TRUE, 227, 4, NULL, skin, NULL);
		add_button("stop", (gchar **)btn_mstop_xpm, TRUE, TRUE, 243, 4, NULL, skin, NULL);
		add_button("prev", (gchar **)btn_mprev_xpm, TRUE, FALSE, 264, 4, NULL, skin, NULL);
		add_button("next", (gchar **)btn_mnext_xpm, TRUE, FALSE, 280, 4, NULL, skin, NULL);

		add_button("fullscreen", (gchar **)btn_mode_xpm, TRUE, FALSE, 301, 4, NULL, skin, NULL);
		add_button("fullscreen.controls", (gchar **)btn_exit_xpm, TRUE, FALSE, 317, 4, NULL, skin, NULL);

		ds = add_digit("small",(gchar **)dig_lrg_xpm, skin, NULL);

		add_number("hour", 1, 0, FALSE, FALSE, 137, 4, ds, skin, NULL);
		add_number("minute", 2, 0, TRUE, FALSE, 153, 4, ds, skin, NULL);
		add_number("second", 2, 0, TRUE, FALSE, 181, 4, ds, skin, NULL);

		finish_simple_window(skin);
		}
	else
		{
		/* main mode */
		skin->real_overlay = util_size_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)back_xpm), TRUE);

		skin->width_def = skin->width = 354;
		skin->height_def = skin->height = 321;

		skin->sizeable = TRUE;
		skin->width_min = 322;
		skin->height_min = 145;
		skin->width_max = 754;
		skin->height_max = 593;

		skin->width_inc = 16;
		skin->height_inc = 16;

		skin->has_border = TRUE;
		skin->border_left = 126;
		skin->border_right = 190;
		skin->border_top = 60;
		skin->border_bottom = 66;

		add_text("title", GQOOB_SKIN_FONT, 215, TRUE, 0, 255, 255, 255, 179, 82, 6, skin, NULL);

		wd = add_button("pause", (gchar **)btn_pause_xpm, TRUE, TRUE, 10, 281, NULL, skin, NULL);
		ui_widget_set_anchor(wd, FALSE, TRUE);
		wd = add_button("play", (gchar **)btn_play_xpm, TRUE, TRUE, 50, 281, NULL, skin, NULL);
		ui_widget_set_anchor(wd, FALSE, TRUE);
		wd = add_button("stop", (gchar **)btn_stop_xpm, TRUE, TRUE, 90, 281, NULL, skin, NULL);
		ui_widget_set_anchor(wd, FALSE, TRUE);
		wd = add_button("prev", (gchar **)btn_prev_xpm, TRUE, FALSE, 165, 291, NULL, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, TRUE);
		wd = add_button("next", (gchar **)btn_next_xpm, TRUE, FALSE, 200, 291, NULL, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, TRUE);

		wd = add_button("fullscreen", (gchar **)btn_mode_xpm, TRUE, FALSE, 302, 4, NULL, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, FALSE);
		wd = add_button("iconify", (gchar **)btn_iconify_xpm, TRUE, FALSE, 318, 4, NULL, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, FALSE);
		wd = add_button("exit", (gchar **)btn_exit_xpm, TRUE, FALSE, 334, 4, NULL, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, FALSE);

		wd = add_button("skin_size", (gchar **)btn_sizer_xpm, TRUE, FALSE, 330, 297, NULL, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, TRUE);

		ds = add_digit("small",(gchar **)dig_sml_xpm, skin, NULL);
		dl = add_digit("large",(gchar **)dig_lrg_xpm, skin, NULL);

		wd = add_number("hour", 2, 0, FALSE, FALSE, 255, 271, dl, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, TRUE);
		wd = add_number("minute", 2, 0, TRUE, FALSE, 283, 271, dl, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, TRUE);
		wd = add_number("second", 2, 0, TRUE, FALSE, 311, 271, dl, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, TRUE);

		wd = add_number("song", 3, 0, FALSE, TRUE, 274, 288, ds, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, TRUE);
		wd = add_number("song_total", 3, 0, FALSE, TRUE, 305, 288, ds, skin, NULL);
		ui_widget_set_anchor(wd, TRUE, TRUE);

		overlay_register_to_skin("video", 17, 29, 320, 240, TRUE, skin, NULL);

		skin_finalize(skin);
		}

	add_focus(skin);
	double_focus(skin);

	return skin;
}

SkinData *skin_load_default_flyby_cb(UIData *ui, const gchar *key, gpointer data)
{
	SkinData *skin;

	skin = skin_new();

	skin->real_overlay = util_size_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)flyby_back_xpm), TRUE);

	add_text("title", GQOOB_SKIN_FONT, 236, FALSE, 0, 0, 0, 0, 179, 7, 7, skin, NULL);
	add_text("artist", GQOOB_SKIN_FONT, 206, FALSE, 0, 0, 0, 0, 179, 7, 25, skin, NULL);

	add_button("next", (gchar **)btn_snext_xpm, TRUE, FALSE, 231, 30, NULL, skin, NULL);
	add_button("prev", (gchar **)btn_sprev_xpm, TRUE, FALSE, 218, 30, NULL, skin, NULL);

	skin->width = 250;
	skin->height = 48;

	skin->has_border = TRUE;
	skin->border_left = 8;
	skin->border_right = 8;
	skin->border_top = 8;
	skin->border_bottom = 8;

	/* this is merely to that the skin editor has sane values for these */
	skin->width_def = skin->width_min = skin->width_max = skin->width;
	skin->height_def = skin->height_min = skin->height_max = skin->height;

	return skin;
}

SkinData *skin_load_default_menu_cb(UIData *ui, const gchar *key, gpointer data)
{
	SkinData *skin;
	ListData *ld;
	WidgetData *wd;

	skin = skin_new();

	skin->real_overlay = util_size_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)menu_back_xpm), TRUE);
	skin->has_border = TRUE;
	skin->border_left = 6;
	skin->border_right = 6;
	skin->border_top = 6;
	skin->border_bottom = 6;
	skin->width = 25;
	skin->height = 24;

	ld = list_new(gdk_pixbuf_new_from_xpm_data((const char **)menu_list_back_xpm),
		      3, 3, 19, 18, TRUE, 2,
		      0, 0, 0, 0, FALSE);
	list_image_row(ld, gdk_pixbuf_new_from_xpm_data((const char **)menu_list_row_xpm),
		       TRUE, TRUE, 2, 2, FALSE, NULL);
	list_image_row_flag(ld, gdk_pixbuf_new_from_xpm_data((const char **)menu_list_icons_xpm), 4, 0);
	list_set_font(ld, GQOOB_SKIN_FONT, NULL, FALSE, 0, 0, 0, 179);

	wd = list_register(skin, ld, "menu", NULL);
	
	if (key && strcmp(key, "skindata_submenu") == 0)
		{
		list_set_column_attributes(ld, 0, 9, UI_LIST_COLUMN_SIZE_FIXED, "icon");
		list_set_column_attributes(ld, 1, 16, 0, "text");
		}
	else
		{
		list_set_column_attributes(ld, 0, 16, 0, "text");
		list_set_column_attributes(ld, 1, 9, UI_LIST_COLUMN_SIZE_FIXED, "icon");
		ld->flag_column = 1;
		ld->row_border_left += 3;
		list_set_column_justify(ld, TRUE);

		ui_widget_set_data(wd, "data", "skindata_submenu");
		}

	double_window(skin);

	/* this is merely to that the skin editor has sane values for these */
	skin->width_def = skin->width_min = skin->width_max = skin->width;
	skin->height_def = skin->height_min = skin->height_max = skin->height;

	/* focus */
	skin->focus_overlay = util_size_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)focus_xpm), TRUE);
	skin->focus_box_alpha = 128;
	skin->focus_has_border = TRUE;
	skin->focus_border_left = 5;
	skin->focus_border_right = 5;
	skin->focus_border_bottom = 5;
	skin->focus_border_top = 5;

	return skin;
}

