/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "ui2_includes.h"
#include "ui2_typedefs.h"

#include "ui2_overlay.h"
#include "ui2_overlay_edit.h"

#include "ui2_editor.h"
#include "ui2_widget.h"
#include "ui_fileops.h"
#include "ui_tabcomp.h"
#include "ui_utildlg.h"


typedef struct _OverlayDetail OverlayDetail;
struct _OverlayDetail
{
	GtkWidget *width_spin;
	GtkWidget *height_spin;
	GtkWidget *dynamic_button;

	OverlayData *overlay;
	WidgetData *wd;
	UIData *ui;
};

typedef struct _OverlayListData OverlayListData;
struct _OverlayListData
{
	gchar *key;
	gchar *text_id;
	gchar *data;

	/* this is a filler struct, merely to keep similar workings as other widgets */
};

typedef struct _OverlayPage OverlayPage;
struct _OverlayPage
{
	GtkWidget *key_entry;
	GtkWidget *text_id_entry;
	GtkWidget *data_entry;

	GtkWidget *width_spin;
	GtkWidget *height_spin;
	GtkWidget *dynamic_button;

	GtkWidget *clist;
	GtkWidget *pixmap;

	EditData *ed;
};


static void overlay_edit_write(FILE *f, WidgetData *wd, SkinData *skin, const gchar *dir)
{
	OverlayData *od = wd->widget;

	ui_edit_write_section(f, "overlay", wd->key);

	ui_edit_write_key_int(f, "x", od->x);
	ui_edit_write_key_int(f, "y", od->y);
	ui_edit_write_key_int(f, "width", od->width);
	ui_edit_write_key_int(f, "height", od->height);
	
	ui_edit_write_key_bool(f, "dynamic", od->dynamic);
}

#if 0
static gpointer overlay_edit_read(UIData *ui, WidgetData *wd, GList *list)
{
	OverlayData *od = wd->widget;
	OverlayListData *ld;

	/* do test for uniqeness here, return NULL if not */

	ld = g_new0(OverlayListData, 1);

	ld->key = g_strdup(wd->key);
	ld->data = g_strdup(ui_widget_get_data(wd, "data"));
	ld->text_id = g_strdup(wd->text_id);

	return ld;
}
#endif

static void overlay_edit_props_size_cb(GtkAdjustment *adjustment, gpointer data)
{
	OverlayDetail *dd = data;
	OverlayData *overlay;
	gint new_w, new_h;
	gint cw, ch;

	overlay = dd->overlay;

	new_w = ui_edit_spin_get(dd->width_spin);
	new_h = ui_edit_spin_get(dd->height_spin);

	if (new_w > dd->ui->skin->width)
		{
		new_w = dd->ui->skin->width;
		ui_edit_spin_set_blocking(dd->width_spin, new_w, dd);
		}
	if (overlay->x + new_w > dd->ui->skin->width)
		{
		ui_widget_set_coord(dd->ui, dd->wd, dd->ui->skin->width - new_w, overlay->y, TRUE);
		}

	if (new_h > dd->ui->skin->height)
		{
		new_h = dd->ui->skin->height;
		ui_edit_spin_set_blocking(dd->height_spin, new_h, dd);
		}
	if (overlay->y + new_h > dd->ui->skin->height)
		{
		ui_widget_set_coord(dd->ui, dd->wd, overlay->x, dd->ui->skin->height - new_h, TRUE);
		}

	cw = overlay->width - new_w;
	ch = overlay->height - new_h;

	if (cw == 0 && ch == 0) return;
	if (cw > 0 || ch > 0) ui_edit_widget_draw_highlight(dd->ui, dd->wd, FALSE);

	overlay->width = new_w;
	overlay->height = new_h;

	ui_edit_widget_resync(dd->ui, dd->wd, TRUE, cw, ch);
}

static void overlay_edit_props_dynamic_cb(GtkWidget *widget, gpointer data)
{
	OverlayDetail *dd = data;

	dd->overlay->dynamic = ui_edit_toggle_get(dd->dynamic_button);
}

static gpointer overlay_edit_props(UIData *ui, WidgetData *wd, GtkWidget *vbox, gpointer detail)
{
	OverlayData *od = wd->widget;
	OverlayDetail *dd = detail;

	if (!dd)
		{
		GtkObject *adj;

		dd = g_new0(OverlayDetail, 1);

		dd->width_spin = ui_edit_spin_new(vbox, _("Width:"), 1, 2000, &adj);
		gtk_signal_connect(adj, "value_changed",
				   GTK_SIGNAL_FUNC(overlay_edit_props_size_cb), dd);
		dd->height_spin = ui_edit_spin_new(vbox, _("Height:"), 1, 2000, &adj);
		gtk_signal_connect(adj, "value_changed",
				   GTK_SIGNAL_FUNC(overlay_edit_props_size_cb), dd);
		dd->dynamic_button = ui_edit_toggle_new(vbox, _("Dynamic:"));
		gtk_signal_connect(GTK_OBJECT(dd->dynamic_button), "clicked",
				   GTK_SIGNAL_FUNC(overlay_edit_props_dynamic_cb), dd);
		}

	dd->overlay = od;
	dd->wd = wd;
	dd->ui = ui;

	ui_edit_spin_set_blocking(dd->width_spin, od->width, dd);
	ui_edit_spin_set_blocking(dd->height_spin, od->height, dd);
	ui_edit_toggle_set_blocking(dd->dynamic_button, od->dynamic, dd);

	return dd;
}

#if 0
static void overlay_edit_free(gpointer data)
{
	OverlayListData *ld = data;
	g_free(ld->key);
	g_free(ld->text_id);
	g_free(ld->data);
	g_free(ld);
}
#endif

static void overlay_edit_page_add_cb(GtkWidget *widget, gpointer data)
{
	OverlayPage *pd = data;
	OverlayData *overlay;
	const gchar *key;
	const gchar *text_id;

	key = ui_edit_entry_get(pd->key_entry);
	text_id = ui_edit_entry_get(pd->text_id_entry);

	overlay = overlay_new(0, 0,
			      ui_edit_spin_get(pd->width_spin),
			      ui_edit_spin_get(pd->height_spin),
			      ui_edit_toggle_get(pd->dynamic_button));
	if (overlay)
		{
		WidgetData *wd;
		wd = overlay_register(pd->ed->ui->skin, overlay, key, text_id);
		ui_edit_widget_add_finish(pd->ed, wd, NULL, ui_edit_entry_get(pd->data_entry));
		}
}

static void overlay_edit_page_sync(OverlayPage *pd, OverlayListData *ld)
{
	if (!ld) return;

	ui_edit_entry_set(pd->key_entry, ld->key);
	ui_edit_entry_set(pd->text_id_entry, ld->text_id);
	ui_edit_entry_set(pd->data_entry, ld->data);
}

static void overlay_edit_page_clist_cb(GtkWidget *clist, gint row, gint col, GdkEvent *event, gpointer data)
{
	OverlayPage *pd = data;
	OverlayListData *ld;

	ld = gtk_clist_get_row_data(GTK_CLIST(clist), row);
	overlay_edit_page_sync(pd, ld);
}

static void overlay_edit_page_destroy_cb(GtkWidget *widget, gpointer data)
{
	OverlayPage *pd = data;

	g_free(pd);
}

static GtkWidget *overlay_edit_page_new(EditData *ed)
{
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *item;
	OverlayPage *pd;
	gchar *titles[] = { "", "", NULL };

	pd = g_new0(OverlayPage, 1);
	pd->ed = ed;

	hbox = gtk_hbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
	gtk_object_set_data(GTK_OBJECT(hbox), "page", pd);
	gtk_signal_connect(GTK_OBJECT(hbox), "destroy",
			   GTK_SIGNAL_FUNC(overlay_edit_page_destroy_cb), pd);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
	gtk_widget_show(vbox);

	pd->key_entry = ui_edit_key_entry_new(vbox, ed->ui, overlay_type_id());
	pd->data_entry = ui_edit_entry_new(vbox, _("Data:"));
	pd->text_id_entry = ui_edit_entry_new(vbox, _("Text id:"));

	pd->width_spin = ui_edit_spin_new(vbox, _("Width:"), 1, 2000, NULL);
	pd->height_spin = ui_edit_spin_new(vbox, _("Height:"), 1, 2000, NULL);
	pd->dynamic_button = ui_edit_toggle_new(vbox, _("Dynamic"));

	item = gtk_button_new_with_label(_("Add"));
	gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(item), "clicked",
			   GTK_SIGNAL_FUNC(overlay_edit_page_add_cb), pd);
	gtk_widget_show(item);

	pd->clist = ui_edit_clist_new(hbox, titles, 2);
	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
			   GTK_SIGNAL_FUNC(overlay_edit_page_clist_cb), pd);

	gtk_widget_show(hbox);

	return hbox;
}

static void overlay_edit_page_add(GtkWidget *widget, gpointer data)
{
	OverlayListData *ld = data;
	OverlayPage *pd;
	gchar *buf[4];

	pd = gtk_object_get_data(GTK_OBJECT(widget), "page");

	buf[0] = "";
	buf[1] = ld->key;
	buf[2] = NULL;

	ui_edit_clist_append(pd->clist, buf, NULL, ld);
}

void overlay_type_init_edit(WidgetObjectData *od)
{
	od->func_edit_write = overlay_edit_write;

#if 0
	od->func_edit_read = overlay_edit_read;
	od->func_edit_free = overlay_edit_free;
#endif

	od->func_edit_props = overlay_edit_props;

	od->func_edit_page_new = overlay_edit_page_new;
	od->func_edit_page_add = overlay_edit_page_add;
}

